﻿using System;
using System.Text;
using RestSharp;
using Newtonsoft.Json.Linq;

namespace DigiLockerAuth
{
    public interface IDigiLocker
    {
        string getAccessToken();
        string getRefreshToken();
        string revokeToken(String accessToken);
        string getUserDetails(String accessToken);
        string getListOfIssuedDocuments(String accessToken);
        string getListOfUploadedDocuments(String accessToken);
    }
    public class DigiLockerSDK : IDigiLocker
    {
        private String codeReceived;
        private String clientId;
        private String clientSecret;
        private String accessToken; //try using the parsed value from AutomateFetchAccessToken()
        private String refreshToken; //try using the parsed value from AutomateFetchRefreshToken()
        private String credentials = Convert.ToBase64String(Encoding.ASCII.GetBytes("2DE2CF9A:80e9a8dfba9a8903dff1"));
        private String contentType = "application/x-www-form-urlencoded";
        private String fileId = "";


        /*GETTER AND SETTERS*/
        public void setCodeReceived(String cReceived)
        {
            codeReceived = cReceived;
        }
        public void setClientId(String cId)
        {
            clientId = cId;
        }
        public void setClientSecret(String cSecret)
        {
            clientSecret = cSecret;
        }

        public String getClientId()
        {
            return clientId;
        }
        public String getClientSecret()
        {
            return clientSecret;
        }
        /*===============================================*/

        /*SDK Part ~ Calling The Authorized Partner APIs*/
        public String getAccessToken()
        {
            JObject jObjectbody = new JObject();
            RestClient restClient = new RestClient("https://api.digitallocker.gov.in");

            Console.WriteLine("Request from client: " + clientId);

            jObjectbody.Add("code", codeReceived);
            jObjectbody.Add("client_id", clientId);
            jObjectbody.Add("client_secret", clientSecret);
            jObjectbody.Add("grant_type", "authorization_code");
            jObjectbody.Add("redirect_uri", "http://localhost:9286/index.html");

            RestRequest restRequest = new RestRequest("/public/oauth2/1/token", Method.POST);

            //Adding Json body as parameter to the post request
            restRequest.AddParameter("application/json", jObjectbody, ParameterType.RequestBody);

            IRestResponse restResponse = restClient.Execute(restRequest);



            return restResponse.Content;
        }

        public string getRefreshToken()
        {
            refreshToken = AutomateDigiLockerSDK.AutomateFetchRefreshToken();
            var restClient = new RestClient("https://api.digitallocker.gov.in");

            var restRequest = new RestRequest("/public/oauth2/1/token", Method.POST);


            restRequest.AddHeader("Authorization", "Basic " + credentials);
            restRequest.AddHeader("Content-Type", contentType);


            restRequest.AddParameter("refresh_token", refreshToken);
            restRequest.AddParameter("grant_type", "refresh_token");


            var restResponse = restClient.Execute(restRequest);

            Console.WriteLine("Response StatusCode: " + restResponse.StatusCode);

            Console.WriteLine("Response from RefreshAccessToken API: " + restResponse.Content);

            /* String resultantAccessToken = JObject.Parse("access_token").ToString();
             Console.WriteLine("testParseJson: " + resultantAccessToken);  Try this at home today!!! */

            return restResponse.Content;
        }

        public string revokeToken(String accessToken)
        {

            var restClient = new RestClient("https://api.digitallocker.gov.in");

            var restRequest = new RestRequest("/public/oauth2/1/revoke", Method.POST);


            restRequest.AddHeader("Authorization", "Basic " + credentials);
            restRequest.AddHeader("Content-Type", contentType);


            restRequest.AddParameter("token", accessToken);
            restRequest.AddParameter("token_type_hint", "access_token");


            var restResponse = restClient.Execute(restRequest);

            Console.WriteLine("Response StatusCode: " + restResponse.StatusCode);

            Console.WriteLine("Response from RevokeToken API: " + restResponse.Content);

            return restResponse.Content;
        }

        public string getUserDetails(String accessToken)
        {
            var restClient = new RestClient("https://api.digitallocker.gov.in");

            var restRequest = new RestRequest("/public/oauth2/1/user", Method.GET);

            restRequest.AddHeader("Authorization", "Bearer " + accessToken);

            var restResponse = restClient.Execute(restRequest);


            Console.WriteLine("Response StatusCode: " + restResponse.StatusCode);

            Console.WriteLine("Response from GetUserDetails API: " + restResponse.Content);

            return restResponse.Content;
        }

        public string getListOfIssuedDocuments(String accessToken)
        {

            var restClient = new RestClient("https://api.digitallocker.gov.in");

            var restRequest = new RestRequest("/public/oauth2/2/files/issued", Method.GET);

            restRequest.AddHeader("Authorization", "Bearer " + accessToken);

            var restResponse = restClient.Execute(restRequest);


            Console.WriteLine("Response StatusCode: " + restResponse.StatusCode);

            Console.WriteLine("Response from GetListOfIssuedDocuments API: " + restResponse.Content);


            return restResponse.Content;
        }
        public string getListOfUploadedDocuments(String accessToken)
        {

            var restClient = new RestClient("https://api.digitallocker.gov.in");

            var restRequest = new RestRequest("/public/oauth2/1/files/" + fileId, Method.GET);


            restRequest.AddHeader("Authorization", "Bearer " + accessToken);


            IRestResponse restResponse = restClient.Execute(restRequest);


            Console.WriteLine("Response StatusCode: " + restResponse.StatusCode);

            Console.WriteLine("Response from GetListOfUploadedDocuments API: " + restResponse.Content);

            return restResponse.Content;
        }
        public static object getDocumentPDF()
        {
            return null;
        }

        public static object getDocumentXML()
        {
            return null;
        }

        public static void uploadDocument(object doc)
        {
        }

        public static void pullDocument(object doc)
        {
        }
        /*===============================++*/
    }
}